module.exports = function(grunt) {

  // Project configuration.
  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    browserify: {
      vendor: {
        src: [],
        dest: 'dist/intellisense-1.0.10-vendor-lib.js',
        options: {
          require: ['lodash-node']
        }
      },
      client: {
        src: ['src/**/*.js'],
        dest:'dist/intellisense-1.0.10.js',
        options: {
          // transform: ['strictify']
          external:['lodash-node']
        }
      }
    },
    uglify: {
      minifyForBrowserify: {
        files: {
          'dist/intellisense-1.0.10.min.js': ['dist/intellisense-1.0.10.js'],
          'dist/intellisense-1.0.10-vendor-lib.min.js': ['dist/intellisense-1.0.10-vendor-lib.js']
        }
      },
      minifyForNPM: {
        files: {
          'dist/intellisense-optimized-minified.js': ['dist/intellisense-optimized.js']
        }
      }
    },
    node_optimize: {
      dist: {
        options: {
          ignore: [
            'dist/*.js',
            'test/*.js'
          ]
        },
        files: {
          'dist/intellisense-optimized.js': 'src/intellisense.js'
        }
      }
    },
    copy: {
      npmPkgCopy: {
        files: [{
          src: 'dist/intellisense-optimized-minified.js',
          dest: 'publish/intellisense-js/intellisense.js'
        }, {
          src: 'package.json',
          dest: 'publish/intellisense-js/package.json'
        }, {
          src: 'README-NPM.md',
          dest: 'publish/intellisense-js/README.md'
        }]
      }
    },
    jshint: {
      files: [
        'src/**/*.js',
        '!node_modules/**/*',
      ],
      options: {
        jshintrc: '.jshintrc'
      }
    },
    'http-server': {
      'dev': {
        // the server root directory
        root: '.',
        port: 8888,
        host: "127.0.0.1",
        cache: 0,
        showDir: true,
        autoIndex: true,
        // server default file extension
        ext: "html",
        // run in parallel with other tasks
        runInBackground: false
      }
    },
    tape: {
      options: {
        pretty: true,
        output: 'console'
      },
      files: ['test/*.js']
    },
    jsdoc: {
      dist: {
        src: ['src/*.js', 'src/events/*.js', 'src/entities/**/*.js', 'src/actions/*.js', 'README-external.md'],
        options: {
          destination: 'doc',
          template: "node_modules/grunt-jsdoc/node_modules/ink-docstrap/template",
          configure: "jsdoc.conf.json"
        }
      }
    },
    clean: ['dist/*.js', 'doc', 'publish/*']
  });

  // Load the plugin and tasks that provides unit testing via tap/tape
  grunt.loadNpmTasks('grunt-tape');
  grunt.registerTask('test', ['tape']);
  grunt.registerTask('ci', ['tape:ci']);

  // Load the plugin that provides the "browserify" task.
  grunt.loadNpmTasks('grunt-browserify');

  // Load plugin that provides http-server task
  grunt.loadNpmTasks('grunt-http-server');

  // Load jsDoc3 plugin
  grunt.loadNpmTasks('grunt-jsdoc');

  // jshint
  grunt.loadNpmTasks('grunt-contrib-jshint');

  // uglify for minification
  grunt.loadNpmTasks('grunt-contrib-uglify');

  // load clean plugin
  grunt.loadNpmTasks('grunt-contrib-clean');

  // optimize plugin
  grunt.loadNpmTasks('grunt-node-optimize');

  //load the copy module
  grunt.loadNpmTasks('grunt-contrib-copy');

  // Default task(s).
  grunt.registerTask('default', ['clean', 'lint', 'test', 'browserify', 'uglify:minifyForBrowserify', 'doc']);
  grunt.registerTask('server', ['http-server']);
  grunt.registerTask('doc', ['jsdoc']);
  grunt.registerTask('lint', ['jshint']);
  grunt.registerTask('publish', ['clean', 'lint', 'test', 'node_optimize', 'uglify:minifyForNPM', 'copy:npmPkgCopy']);
  grunt.registerTask('distribute', ['clean', 'lint', 'test', 'node_optimize', 'uglify:minifyForNPM', 'copy:npmPkgCopy', 
    'browserify', 'uglify:minifyForBrowserify', 'doc']);

};
