"use strict";


/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
// var http; // require set in initialize()
var cors = require('./cors');
// var Q = require('q');
var logger = require('./logger');

var sensorOptions = {};

/*
 * Check if client is properly initialized
 */
var initialized = function() {
  return true; //TODO
};

var simpleType = function(forType) {

  var mappedTypes = {
    "http://purl.imsglobal.org/caliper/v1/ActivityContext": "activityContext",
    "http://purl.imsglobal.org/caliper/v1/Agent": "agent",
    "http://purl.imsglobal.org/caliper/v1/Attempt": "attempt",
    "http://purl.imsglobal.org/caliper/v1/DigitalResource": "digitalResource",
    "http://purl.imsglobal.org/caliper/v1/Entity": "entity",
    "http://purl.imsglobal.org/caliper/v1/LearningObjective": "learningObjective",
    "http://purl.imsglobal.org/caliper/v1/Result": "result",
    "http://purl.imsglobal.org/caliper/v1/View": "view",
    "http://purl.imsglobal.org/caliper/v1/Session": "session",
    "http://purl.imsglobal.org/caliper/v1/Assessment": "quiz",
    "http://purl.imsglobal.org/caliper/v1/AssessmentItem": "quizItem",
    "http://purl.imsglobal.org/caliper/v1/Annotation": "annotation",
    "http://purl.imsglobal.org/caliper/v1/BookmarkAnnotation": "bookmarkAnnotation",
    "http://purl.imsglobal.org/caliper/v1/HighlightAnnotation": "highlightAnnotation",
    "http://purl.imsglobal.org/caliper/v1/SharedAnnotation": "sharedAnnotation",
    "http://purl.imsglobal.org/caliper/v1/TagAnnotation": "tagAnnotation",
    "http://purl.imsglobal.org/caliper/v1/lis/Person": "person",
    "http://purl.imsglobal.org/caliper/v1/lis/Organization": "institution",
    "http://purl.imsglobal.org/caliper/v1/lis/CourseSection": "course",
    "http://purl.imsglobal.org/caliper/v1/AudioObject": "audioObject",
    "http://purl.imsglobal.org/caliper/v1/ImageObject": "imageObject",
    "http://purl.imsglobal.org/caliper/v1/VideoObject": "videoObject",
    "http://purl.imsglobal.org/caliper/v1/MediaLocation": "mediaLocation",
    "http://purl.imsglobal.org/caliper/v1/SoftwareApplication": "edApp",
    "http://purl.imsglobal.org/caliper/v1/AssignableDigitalResource": "assignableDigitalResource",
    "http://www.idpf.org/epub/vocab/structure/#chapter": "ePubChapter",
    "http://www.idpf.org/epub/vocab/structure/#part": "ePubPart",
    "http://www.idpf.org/epub/vocab/structure/#subchapter": "ePubSubChapter",
    "http://www.idpf.org/epub/vocab/structure/#volume": "ePubVolume",
    "http://purl.imsglobal.org/caliper/v1/Frame": "frame",
    "http://purl.imsglobal.org/caliper/v1/MediaObject": "mediaObject",
    "http://www.idpf.org/epub/vocab/structure": "ePubStructure",
    "http://purl.imsglobal.org/caliper/v1/WebPage": "webPage",
  };

  var simpleType = mappedTypes[forType];

  if (_.isUndefined(simpleType)) {
    simpleType = "UNKNOWN";
  }

  return simpleType;
};


/**
 * Initializes the default client to use. Uses the socket consumer by default.
 * @param  array $options passed straight to the client
 */
var initialize = function(options) {
  sensorOptions = options;

  if (_.isUndefined(options.testMode)) {
    sensorOptions.entityURI = '/v1custom/entitydata';
    sensorOptions.eventURI = '/v1custom/eventdata';
    sensorOptions.authURI = '/user/apiToken';
    sensorOptions.refreshTokenURI = '/user/refreshApiToken';
    sensorOptions.queryURI = '/intellisearch';
  } else {
    logger.warn("Sensor is enabled in test mode.  You must provide options.path for correct operation");
  }

  if (!_.isUndefined(sensorOptions.ssl) && sensorOptions.ssl) {
    // http = require('https');
    sensorOptions.protocol = "https://";
  } else {
    // http = require('http');
    sensorOptions.protocol = "http://";
  }

  //logger.log('debug', "Initializing sensor with options " + JSON.stringify(
      //sensorOptions));
};

var addIntellifyMetadata = function(intellifyBaseObj) {
  intellifyBaseObj.apiKey = sensorOptions.apiKey;
  intellifyBaseObj.sensorId = sensorOptions.sensorId;
  intellifyBaseObj.correlationId = sensorOptions.correlationId;
};

// var post = function(postOptions, postData) {

//   var headers = {
//     'Content-Type': 'application/json',
//     'Content-Length': postData.length
//   };

//   postOptions = _.merge(postOptions, {
//     headers: headers
//   });

//   var request = http.request(postOptions, function(response) {
//     logger.log('debug', "finished sending request. Response =" + JSON.stringify(response));
//   });

//   request.write(postData);
//   request.end();

// };

var putCors = function(putOptions, putData) {

  cors.put({
    'method': 'put',
    'url': putOptions.protocol + putOptions.host + putOptions.path,
    'credentials': false,
    'data': putData,
    'headers': {
      "Content-Type": "application/json"
    },
    'success': function(data) {
      logger.info("Successfully PUT learning event" + data);
    },
    'error': function() {
      logger.error("learning event PUT FAILED");
    }
  });

};

var postCors = function(postHeader, postOptions, postParams, postData,
  successCallback, errorCallback) {

  var postUrl = postOptions.protocol + postOptions.host + postOptions.path;
  cors.post({
    'method': 'post',
    'url': postUrl,
    'params': postParams,
    'credentials': false,
    'data': postData,
    'headers': postHeader,
    'success': function(data) {
      //logger.log("debug", "Got data from POST : " + " to path " + postUrl + data);
      //logger.log("Calling success callback");
      successCallback(data);
    },
    'error': function(data) {
      logger.error("POST data FAILED " + " to path " + postUrl + ". Reason : " + data);
      //logger.log("debug", "Calling error callbak");
      errorCallback(data);
    }
  });

};

/**
 * Send learning events
 * @param  CaliperEvent $caliperEvent The Learning Event
 * @return boolean                   whether the measure call succeeded
 */
var measure = function(caliperEvent) {
  if (initialized()) {

    // Create eventData envelope
    var eventData = {};
    eventData.event = caliperEvent;

    if (_.isUndefined(caliperEvent.startedAtTime) || (caliperEvent.startedAtTime ===
      0)) {
      var timestamp = new Date();
      eventData.startedAtTime = timestamp.getTime();
      eventData.timestamp = timestamp.getTime();
    } else {
      eventData.timestamp = caliperEvent.startedAtTime;
    }

    addIntellifyMetadata(eventData);

    // send event
    //logger.log('debug', "Sending event data " + JSON.stringify(eventData));

    var postOptions = _.merge(sensorOptions, {
      method: 'PUT',
      path: sensorOptions.eventURI
    });

    var postData = JSON.stringify(eventData);

    // post(postOptions, postData);
    putCors(postOptions, postData);

  } else {
    logger.error("Sensor is not initialized!! Event will not be sent...");
  }
};

/**
 * Describe an entity
 * @param  CaliperEntity $caliperEntity The Caliper Entity we are describing
 * @return boolean            whether the describe call succeeded
 */
var describe = function(caliperEntity) {

  if (initialized()) {

    // Create entityData envelope
    var entityData = {};
    entityData.entityId = caliperEntity['@id'];
    entityData.type = simpleType(caliperEntity['@type']);
    entityData.entity = caliperEntity;

    if (_.isUndefined(caliperEntity.dateModified) || (caliperEntity.dateModified ===
      0)) {
      var timestamp = new Date();
      entityData.dateModified = timestamp.getTime();
      entityData.timestamp = timestamp.getTime();
    } else {
      entityData.timestamp = caliperEntity.dateModified;
    }

    addIntellifyMetadata(entityData);

    // send entityData
    //logger.log('debug', "Sending entityData " + JSON.stringify(entityData));

    var postOptions = _.merge(sensorOptions, {
      method: 'PUT',
      path: sensorOptions.entityURI
    });

    var postData = JSON.stringify(entityData);

    // post(postOptions, postData);
    putCors(postOptions, postData);

  } else {
    logger.error("Sensor is not initialized!! Describe will not be sent...");
  }
};

/**
 * Authenticate at Intellify service using provided credentials and return a token
 * @param  User $user The user (name and password) to login
 * @param long $expiryInSeconds Token expiration duration in seconds
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */
var authenticate = function(user, expiryInSeconds, successCallback,
  errorCallback) {

  if (initialized()) {

    //send user credentials
    /*logger.log('debug', "Sending user credentials to login " + JSON.stringify(
        user));*/

    var postOptions = _.merge(sensorOptions, {
      method: 'POST',
      path: sensorOptions.authURI
    });

    var postData = JSON.stringify(user);

    var postHeader = {};
    postHeader['Content-Type'] = "application/json";

    var postParams = {};
    postParams.expiryInSeconds = expiryInSeconds;
    var successFunction = function(data) {
      //NOTE: We don't get HTTP status code; hence need to check if we got apiToken in the response
      try {
        var res = JSON.parse(data);
        //Check if we got the apiToken
        if (_.isUndefined(res.apiToken)) {
          //logger.log('error in authentication: ' + data);
          errorCallback("Unauthorized");
        } else {
          //We got the token here
          successCallback(res.apiToken);
        }
      } catch (e) {
      //No password in the response, assume unauthorized error
      errorCallback("Unauthorized");
      }

    };

    var errorFunction = function(data) {
      //logger.log('debug', data);
      errorCallback(data);
    };

    postCors(postHeader, postOptions, postParams, postData, successFunction,
      errorFunction);

  } else {
    //logger.log('error',
      //"Sensor is not initialized!! Auth request will not be sent...");
  }
};

/**
 * Refresh token at Intellify service using logged-in token and return a refresh token
 * @param  String $token The authentication token for the login user
 * @param long $expiryInSeconds Token expiration duration in seconds
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */
var refreshToken = function(token, expiryInSeconds, successCallback,
  errorCallback) {

  if (initialized()) {

    //send refresh token
    /*logger.log('debug', "Sending token for refresh " + token));*/

    var postOptions = _.merge(sensorOptions, {
      method: 'POST',
      path: sensorOptions.refreshTokenURI
    });

    var postData = "{}";

    var postHeader = {};
    postHeader['Content-Type'] = "application/json";
    postHeader.Authorization = "Bearer " + token;

    var postParams = {};
    postParams.expiryInSeconds = expiryInSeconds;
    var successFunction = function(data) {
      //NOTE: We don't get HTTP status code; hence need to check if we got apiToken in the response
      try {
        var res = JSON.parse(data);
        //Check if we got the apiToken
        if (_.isUndefined(res.apiToken)) {
          //logger.log('error in refresh token: ' + data);
          errorCallback("Unauthorized");
        } else {
          //We got the token here
          successCallback(res.apiToken);
        }
      } catch (e) {
      //No password in the response, assume unauthorized error
      errorCallback("Unauthorized");
      }

    };

    var errorFunction = function(data) {
      //logger.log('debug', data);
      errorCallback(data);
    };

    postCors(postHeader, postOptions, postParams, postData, successFunction,
      errorFunction);

  } else {
    //logger.log('error',
      //"Sensor is not initialized!! Auth request will not be sent...");
  }
};

/**
 * Query an Intellify stream and return the result
 * @param  String $token The authentication token for the login user
 * @param String $stream Name of the stream to be searched
 * @param boolean $isDynamic True if this is dynamic query
 * @param String $urlParams URL parameters if any
 * @param String $requestBody JSON query string which will be the request body
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */

var query = function(token, stream, isDynamic, urlParams, requestBody, successCallback,
  errorCallback) {

  if (initialized()) {

    //send user credentials
    //logger.log('debug', "Querying stream " + stream);
    //logger.log('debug', "Query URL params " + urlParams);
    //logger.log('debug', "Query request body " + requestBody);

    var postOptions = _.merge(sensorOptions, {
      method: 'POST',
      path: sensorOptions.queryURI + "/" + (isDynamic? "dynamic/" + stream : stream + "/_search"),
    });


    var postHeader = {};
    postHeader['Content-Type'] = "application/json";
    postHeader.Authorization = "Bearer " + token;


    var successFunction = function(data) {
      //NOTE: We don't get HTTP status code;
      //So we look for 'error' in data.
      try {
        var res = JSON.parse(data);
        //Check if we did not get any error
        if (_.isUndefined(res.error)) {
          //We got the valid query response here
          //TODO : Hide ES specific fields from the caller
          //logger.log('debug', "Query result : " + data);
          successCallback(data);
        } else {
          //logger.log('error in query: ' + data);
          errorCallback(data);
        }
      } catch (e) {
      //No password in the response, assume unauthorized error
      errorCallback("Bad query response format");
      }
    };

    var errorFunction = function(data) {
      //logger.log('debug', "Query error " + data);
      errorCallback(data);
    };

    postCors(postHeader, postOptions, urlParams, requestBody, successFunction,
      errorFunction);

  } else {
    //logger.log('error',
      //"Sensor is not initialized!! Auth request will not be sent...");
  }
};

module.exports = {
  initialize: initialize,
  measure: measure,
  describe: describe,
  authenticate: authenticate,
  refreshToken: refreshToken,
  query: query
};
