"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Entity = require('../caliperEntity');

/**
 *
 * Entity type: Annotation
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Annotation
 *
 * Construct as follows:
 * <pre><code>
 * var myAnnotation = new new Intellify.Entities.Annotation("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.Entity
 */
function Annotation(id) {

  Entity.call(this);

  this.setId(id);

  this.setName(null);
  this.setDescription(null);
  this.setProperties({});
}

Annotation.prototype = _.create(Entity.prototype);

Annotation.prototype.setAnnotationType = function(type) {
  switch (type) {
    case "ANNOTATION":
      this.setType("http://purl.imsglobal.org/caliper/v1/Annotation");
      break;
    case "BOOKMARK_ANNOTATION":
      this.setType("http://purl.imsglobal.org/caliper/v1/BookmarkAnnotation");
      break;
    case "HIGHLIGHT_ANNOTATION":
      this.setType("http://purl.imsglobal.org/caliper/v1/HighlightAnnotation");
      break;
    case "SHARED_ANNOTATION":
      this.setType("http://purl.imsglobal.org/caliper/v1/SharedAnnotation");
      break;
    case "TAG_ANNOTATION":
      this.setType("http://purl.imsglobal.org/caliper/v1/TagAnnotation");
      break;
  }
};

/**
 * Enum for JSON-LD types of annotations
 * @readonly
 * @enum {string}
 */
Annotation.prototype.annotationTypes = {
  "BOOKMARK_ANNOTATION": "http://purl.imsglobal.org/caliper/v1/BookmarkAnnotation",
  "HIGHLIGHT_ANNOTATION": "http://purl.imsglobal.org/caliper/v1/HighlightAnnotation",
  "SHARED_ANNOTATION": "http://purl.imsglobal.org/caliper/v1/SharedAnnotation",
  "TAG_ANNOTATION": "http://purl.imsglobal.org/caliper/v1/TagAnnotation"
};


module.exports = Annotation;
