"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var DigitalResource = require('../DigitalResource');

/**
 *
 * Entity type: AssignableDigitalResource 
 *
 * A AssignableDigitalResource is the base type for all digital content that is assignable (e.g. quizzes, homework, etc.)
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/AssignableDigitalResource
 *
 * Construct as follows:
 * <pre><code>
 * var myUser = new new Intellify.Entities.AssignableDigitalResource("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.DigitalResource
 */
function AssignableDigitalResource(id) {

  DigitalResource.call(this);

  this.setId(id);
  this.setType(this.Types.ASSIGNABLE_DIGITAL_RESOURCE);

  this.setProperties({});
  this.setDateCreated(0);
  this.setDateToActivate(0);
  this.setDateToShow(0);
  this.setDateToStartOn(0);
  this.setDateToSubmit(0);
  this.setMaxAttempts(0);
  this.setMaxSubmits(0);
  this.setMaxScore(0);
}

AssignableDigitalResource.prototype = _.create(DigitalResource.prototype);

AssignableDigitalResource.prototype.setDateCreated = function (dateCreated) {
  this.dateCreated = dateCreated;
};

AssignableDigitalResource.prototype.setDateToActivate = function (dateToActivate) {
  this.dateToActivate = dateToActivate;
};

AssignableDigitalResource.prototype.setDateToShow = function (dateToShow) {
  this.dateToShow = dateToShow;
};

AssignableDigitalResource.prototype.setDateToStartOn = function (dateToStartOn) {
  this.dateToStartOn = dateToStartOn;
};

AssignableDigitalResource.prototype.setDateToSubmit = function (dateToSubmit) {
  this.dateToSubmit = dateToSubmit;
};

AssignableDigitalResource.prototype.setMaxAttempts = function (maxAttempts) {
  this.maxAttempts = maxAttempts;
};

AssignableDigitalResource.prototype.setMaxSubmits = function (maxSubmits) {
  this.maxSubmits = maxSubmits;
};

AssignableDigitalResource.prototype.setMaxScore = function (maxScore) {
  this.maxScore = maxScore;
};

/**
 * Enum for JSON-LD types of assignable digital resources
 * @readonly
 * @enum {string}
 */
AssignableDigitalResource.prototype.Types = {
  "ASSESSMENT": "http://purl.imsglobal.org/caliper/v1/Assessment",
  "ASSESSMENT_ITEM": "http://purl.imsglobal.org/caliper/v1/AssessmentItem"
};


module.exports = AssignableDigitalResource;
