"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Entity = require('../caliperEntity');

/**
 *
 * Entity type: Attempt
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Attempt
 *
 * Represents an Attempt (e.g. for a quiz, question, etc.)
 *
 * Construct as follows:
 * <pre><code>
 * var myAttempt = new new Intellify.Entities.Attempt("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.Entity
 */
function Attempt(id) {

  Entity.call(this);

  this.setId(id);
  this.setType(this.Types.ATTEMPT);

  this.setProperties({});
}

Attempt.prototype = _.create(Entity.prototype);

/**
 * Set the the attempt startedAtTime
 *
 * @param {long} startedAtTime
 */
Attempt.prototype.setStartedAtTime = function (startedAt) {
  this.startedAtTime = startedAt;
};

/**
 * Set the the attempt endedAtTime
 *
 * @param {long} endedAtTime
 */
Attempt.prototype.setEndedAtTime = function (endedAt) {
  this.endedAtTime = endedAt;
};

/**
 * Set the the attempt duration
 *
 * @param {long} duration
 */
Attempt.prototype.setDuration = function (duration) {
  this.duration = duration;
};

/**
 * Set the the assignable (for which Result is being reported)
 *
 * @param {string} assignableId
 */
Attempt.prototype.setAssignableId = function (assignableId) {
  this.assignable = assignableId;
};

/**
 * Set the the actor (who's attempt is being reported on by Result)
 *
 * @param {string} actorId
 */
Attempt.prototype.setActorId = function (actorId) {
  this.actor = actorId;
};

/**
 * Set the the attempt count
 *
 * @param {long} count
 */
Attempt.prototype.setCount = function (count) {
  this.count = count;
};

module.exports = Attempt;
