"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Agent = require('../agent');

/**
 *
 * Entity type: Person
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Person
 *
 * Represents an Person (e.g. a user, student)
 *
 * Construct as follows:
 * <pre><code>
 * var myPerson = new new Intellify.Entities.Person("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.Agent
 */
function Person(id) {

  Agent.call(this);

  this.setId(id);
  this.setType("http://purl.imsglobal.org/caliper/v1/lis/Person");

}

Person.prototype = _.create(Agent.prototype);

Person.prototype.setFirstName = function(firstName) {
  this.firstName = firstName;
};

Person.prototype.setLastName = function(lastName) {
  this.lastName = lastName;
};

Person.prototype.setEmail = function(email) {
  this.email = email;
};

Person.prototype.setRoles = function(roles) {
  this.roles = roles;
};

module.exports = Person;
