"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Entity = require('./mediaObject');

/**
 *
 * Entity type: MediaLocation
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/MediaLocation
 *
 * Represents an location with a MediaObject (e.g. frame in a video, etc.)
 *
 * Construct as follows:
 * <pre><code>
 * var myCourseSection = new new Intellify.Entities.MediaLocation("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.MediaObject
 */
function MediaLocation(id) {

  Entity.call(this);

  this.setId(id);
  this.setType(this.Types.MEDIA_LOCATION);

  this.setName(null);
  this.setProperties({});
}

MediaLocation.prototype = _.create(Entity.prototype);

/**
 * Set the time offset from start
 *
 * @param {long} currentTime
 */
MediaLocation.prototype.setCurrentTime = function (currentTime) {
  this.currentTime = currentTime;
};

/**
 * Set the location offset from start
 *
 * @param {long} offset
 */
MediaLocation.prototype.offset = function (offset) {
  this.offset = offset;
};

module.exports = MediaLocation;
