"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var DigitalResource = require('../digitalResource');

/**
 *
 * Entity type: MediaObject
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/MediaObject
 *
 * Represents an MediaObject (this is a base class)
 *
 * Construct as follows:
 * <pre><code>
 * var myCourseSection = new new Intellify.Entities.MediaObject("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.DigitalResource
 */
function MediaObject(id) {

  DigitalResource.call(this);

  this.setId(id);
  this.setType(this.Types.MEDIA_OBJECT);

  this.setProperties({});
}

MediaObject.prototype = _.create(DigitalResource.prototype);

/**
 * Enum for JSON-LD types of media objects
 * @readonly
 * @enum {string}
 */
MediaObject.prototype.Types = {
  "AUDIO_OBJECT": "http://purl.imsglobal.org/caliper/v1/AudioObject",
  "IMAGE_OBJECT": "http://purl.imsglobal.org/caliper/v1/ImageObject",
  "VIDEO_OBJECT": "http://purl.imsglobal.org/caliper/v1/VideoObject",
  "MEDIA_LOCATION": "http://purl.imsglobal.org/caliper/v1/MediaLocation",
};

MediaObject.prototype.setDuration = function (duration) {
  this.duration = duration;
};

module.exports = MediaObject;
