"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Entity = require('../caliperEntity');

/**
 *
 * Entity type: Result
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Result
 *
 * Represents an Result (outcome of an Attempt on a AssignableDigitalResource)
 *
 * Construct as follows:
 * <pre><code>
 * var myCourseSection = new new Intellify.Entities.Result("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.Entity
 */
function Result(id) {

  Entity.call(this);

  this.setId(id);
  this.setType(this.Types.RESULT);

  this.setProperties({});
}

Result.prototype = _.create(Entity.prototype);

/**
 * Set the the actor (who's attempt is being reported on by Result)
 *
 * @param {string} actorId
 */
Result.prototype.setActorId = function (actorId) {
  this.actor = actorId;
};

/**
 * Set the the assignable (for which Result is being reported)
 *
 * @param {string} assignableId
 */
Result.prototype.setAssignableId = function (assignableId) {
  this.assignable = assignableId;
};

/**
 * Set the the lexile measure
 *
 * @param {long} lexileMeasure
 */
Result.prototype.setLexileMeasure = function (lexileMeasure) {
  this.lexileMeasure = lexileMeasure;
};

/**
 * Set the the normalScore
 *
 * @param {long} normalScore
 */
Result.prototype.setNormalScore = function (normalScore) {
  this.normalScore = normalScore;
};

/**
 * Set the the penaltyScore
 *
 * @param {long} penaltyScore
 */
Result.prototype.setPenaltyScore = function (penaltyScore) {
  this.penaltyScore = penaltyScore;
};

/**
 * Set the the extraCreditScore
 *
 * @param {long} extraCreditScore
 */
Result.prototype.setExtraCreditScore = function (extraCreditScore) {
  this.extraCreditScore = extraCreditScore;
};

/**
 * Set the the totalScore
 *
 * @param {long} totalScore
 */
Result.prototype.setTotalScore = function (totalScore) {
  this.totalScore = totalScore;
};

/**
 * Set the the curvedTotalScore
 *
 * @param {long} curvedTotalScore
 */
Result.prototype.setCurvedTotalScore = function (curvedTotalScore) {
  this.curvedTotalScore = curvedTotalScore;
};

/**
 * Set the the curveFactor
 *
 * @param {long} curveFactor
 */
Result.prototype.setCurveFactor = function (curveFactor) {
  this.curveFactor = curveFactor;
};

/**
 * Set any comments by scoree
 *
 * @param {string} comment
 */
Result.prototype.setComment = function (comment) {
  this.comment = comment;
};

/**
 * Set who scored tihs
 *
 * @param {Agent} scoredBy
 */
Result.prototype.setScoredBy = function (scoredBy) {
  this.scoredBy = scoredBy;
};

module.exports = Result;
