"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var DigitalResource = require('../digitalResource');

/**
 *
 * Entity type: Frame
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Frame
 *
 * Represents an Frame (base type for location within a DigitalResource. E.g. web page location)
 *
 * Construct as follows:
 * <pre><code>
 * var myCourseSection = new new Intellify.Entities.Frame("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Entities.DigitalResource
 */
function Frame(id) {

  DigitalResource.call(this);

  this.setId(id);

  this.setName(null);
  this.setObjectType([]);
  this.setAlignedLearningObjective([]);
  this.setKeywords([]);
  this.setIsPartOf(null);
}

Frame.prototype = _.create(DigitalResource.prototype);

/**
 * Set the the index (location identifier)
 *
 * @param {long} index 
 */
Frame.prototype.setIndex = function (index) {
  this.index = index;
};

module.exports = Frame;
