"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var Entity = require('../caliperEntity');

/**
 *
 * Entity type: Session
 * @extends Intellify.Entities.Entity
 *
 * JSON-LD type: http://purl.imsglobal.org/caliper/v1/Session
 *
 * Construct as follows:
 * <pre><code>
 * var mySession = new new Intellify.Entities.Session("ID_or_URI");
 * </pre></code>
 * @constructor
 */
function Session(id) {

    Entity.call(this);

    this.setId(id);
    this.setType(this.Types.SESSION);

    this.setProperties({});
}

Session.prototype = _.create(Entity.prototype);

/**
 * Set the session actor (typically a user)
 *
 * @param {object} actor
 */
Session.prototype.setActor = function(actor) {
    this.actor = actor;
};

/**
 * Set the session started at time
 *
 * @param {string} startedAtTime
 */
Session.prototype.setStartedAtTime = function(startedAt) {
    this.startedAtTime = startedAt;
};

/**
 * Set the session ended at time
 *
 * @param {string} endedAtTime
 */
Session.prototype.setEndedAtTime = function(endedAt) {
    this.endedAtTime = endedAt;
};

/**
 * Set the session duration
 *
 * @param {string} duration
 */
Session.prototype.setDuration = function(duration) {
    this.duration = duration;
};

module.exports = Session;
