"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var LearningEvent = require('./caliperEvent');

/**
 *
 * Entity type: NavigationEvent
 *
 * JSON-LD type: http://purl.imsglobal.org/ctx/caliper/v1/NavigationEvent
 *
 * Represents an NavigationEvent 
 *
 * Construct as follows:
 * <pre><code>
 * var event = new new Intellify.Events.NavigationEvent("ID_or_URI");
 * </pre></code>
 * @constructor
 * @extends Intellify.Events.Event
 */
function NavigationEvent() {

  LearningEvent.call(this);

  this.setContext(this.Contexts.NAVIGATION);
  this.setType(this.Types.NAVIGATION);

  this.setGenerated(null);
  this.setStartedAtTime(0);
  this.setEndedAtTime(0);
  this.setDuration(null);
}

NavigationEvent.prototype = _.create(LearningEvent.prototype);

NavigationEvent.prototype.setNavigatedFrom = function (navigatedFrom) {
  this.navigatedFrom = navigatedFrom;
};

module.exports = NavigationEvent;
